"use strict";

module.exports = function (grunt) {

	grunt.initConfig({
		// read in the project settings from the package.json file into the pkg property
		pkg: grunt.file.readJSON("package.json"),
		dirs: {
			nodeModules: "node_modules",
			src: "src",
			tmp: "tmp",
			// output
			build: "build"
		},
		html2js: {
			options: {
				base: "app",
				module: "templates",
				// add angular dependency on `templates`
				singleModule: true,
				standalone: true,
				useStrict: true,
				rename: function (moduleName) {
					return moduleName.substring(moduleName.lastIndexOf("/") + 1, moduleName.length);
				},
				htmlmin: {
					collapseWhitespace: true,
					removeComments: true,
				}
			},
			main: {
				src: ["<%= dirs.src %>/**/*.html"],
				dest: "<%= dirs.tmp %>/templates.js"
			}
		},
		concat: {
			javascript: {
				src: [
					"<%= dirs.src %>/**/*.js",
					"!<%= dirs.src %>/**/*tests.js",
					"<%= dirs.tmp %>/**/*.js"
				],
				dest: "<%= dirs.build %>/<%= pkg.name %>.js",
				nonull: true
			},
			css: {
				src: [
					"<%= dirs.src %>/**/*.css",
				],
				dest: "<%= dirs.build %>/<%= pkg.name %>.css",
				nonull: true
			}
		},
		ngAnnotate: {
			options: {
				singleQuotes: false
			},
			app: {
				files: {
					"<%= dirs.build %>/<%= pkg.name %>.js": ["<%= dirs.build %>/<%= pkg.name %>.js"]
				}
			}
		},
		uglify: {
			options: {
				compress: {
					properties: true,
					drop_console: true
				},
				maxLineLen: 5000
			},
			app: {
				options: {
					sourceMap: true,
					sourceMapIncludeSources: false
				},
				files: {
					"<%= dirs.build %>/<%= pkg.name %>.min.js": ["<%= dirs.build %>/<%= pkg.name %>.js"]
				}
			}
		},
		cssmin: {
			options: {
				shorthandCompacting: false,
				roundingPrecision: -1
			},
			target: {
				files: {
					"<%= dirs.build %>/<%= pkg.name %>.min.css": ["<%= dirs.build %>/<%= pkg.name %>.css"]
				}
			}
		},
		jshint: {
			files: ["<%= dirs.src %>/**/*.js"],
			options: {
				globalstrict: true,
				globals: {
					console: false,
					angular: false,
					module: false,
					expect: false,
					describe: false,
					beforeEach: false,
					"$": false,
					inject: false,
					it: false
				}
			}
		},
		karma: {
			unit: {
				options: {
					frameworks: ['jasmine'],
					singleRun: true,
					browsers: ['PhantomJS'],
					files: [
						'node_modules/jquery/dist/jquery.js',
						'node_modules/bootstrap/dist/js/bootstrap.js',
						'node_modules/angular/angular.js',
						'node_modules/angular-mocks/angular-mocks.js',
						'tmp/**/*.js',
						'src/**/*.js'
					]
				},
				reporters: [
					'progress',
					'coverage'
				],
				preprocessors: {
					'src/**/!(*tests).js': [
						'coverage'
					]
				},
				coverageReporter: {
					dir: 'coverage/',
					reporters: [
						// reporters not supporting the `file` property
						{
							type: 'html',
							subdir: 'report-html'
						}, {
							type: 'text-summary'
						}, {
							type: 'text',
							subdir: '.',
							file: 'text.txt'
						}, {
							type: 'text-summary',
							subdir: '.',
							file: 'text-summary.txt'
						},
					]
				},
				//logLevel: 'DEBUG'
			}
		},
		clean: {
			options: {
				"force": true,
				"no-write": false
			},
			tmp: ["<%= dirs.tmp %>/*"]
		},
		'http-server': {
			'dev': {

				// the server root directory
				root: ".",

				// the server port
				// can also be written as a function, e.g.
				// port: function() { return 8282; }
				port: 8282,

				// the host ip address
				// If specified to, for example, "127.0.0.1" the server will
				// only be available on that ip.
				// Specify "0.0.0.0" to be available everywhere
				host: "0.0.0.0",

				//cache: <sec>,
				showDir: true,
				autoIndex: true,

				// server default file extension
				ext: "html",

				// run in parallel with other tasks
				runInBackground: false,

				// specify a logger function. By default the requests are
				// sent to stdout.
				logFn: function (req, res, error) {},

				// Tell grunt task to open the browser
				openBrowser: true,

			}
		},
		copy: {
			main: {
				src: 'build/*',
				dest: 'samples/',
			},
		},

	});

	grunt.loadNpmTasks("grunt-contrib-clean");
	grunt.loadNpmTasks("grunt-contrib-copy");
	grunt.loadNpmTasks("grunt-contrib-concat");
	grunt.loadNpmTasks("grunt-html2js");
	grunt.loadNpmTasks("grunt-ng-annotate");
	grunt.loadNpmTasks("grunt-contrib-uglify");
	grunt.loadNpmTasks("grunt-contrib-cssmin");
	grunt.loadNpmTasks('grunt-karma');
	grunt.loadNpmTasks('grunt-contrib-jshint');
	grunt.loadNpmTasks('grunt-http-server');

	grunt.registerTask("clean", ["clean"]);

	grunt.registerTask("server", ["http-server"]);

	grunt.registerTask('test', ['jshint', 'karma']);

	grunt.registerTask('build', ['jshint', "html2js", "concat", "ngAnnotate", "uglify", "cssmin", "copy"]);

	grunt.registerTask("default", ['test', "build"]);
};
