﻿"use strict";
describe('Input Numeric: component', function () {

    var parentScope, controller, element;
    var initMin = 1;
    var initMax = 255;
    var initStep = 2;
    var initNumberOfDec = 0;
    var initReadOnly = false;
    var initValue = 5;
    var validTemplate =
        '<input-numeric min="min" max="max" step="step" number-of-decimals="numberOfDecimals" read-only="readOnly" value="value"></input-numeric>';

    beforeEach(angular.mock.module('inputNumeric'));

    beforeEach(inject(function($compile, $rootScope){

        parentScope = $rootScope.$new();
        parentScope.min = initMin;
        parentScope.max = initMax;
        parentScope.step = initStep;
        parentScope.numberOfDecimals = initNumberOfDec;
        parentScope.readOnly = initReadOnly;
        parentScope.value = initValue;

        element = angular.element(validTemplate);
        $compile(element)(parentScope);

        parentScope.$digest();

        controller = element.scope().$$childTail.$ctrl;

    }));

    it('should render', function () {
        var inputfield = element.find("input[type='text']");
        expect(inputfield.length).not.toEqual(0);

        var buttons = element.find("button");
        expect(buttons.length).toEqual(2);
    });

    it('checking initial values', function () {
        expect(controller.min).toEqual(initMin);
        expect(controller.max).toEqual(initMax);
        expect(controller.step).toEqual(initStep);
        expect(controller.numberOfDecimals).toEqual(initNumberOfDec);
        expect(controller.readOnly).toEqual(initReadOnly);
        expect(controller.value).toEqual(initValue);

    });

    it('changing value', function () {
        // decrease value
        controller.decrease();
        expect(controller.value).toEqual(initValue + initStep);
        // increase value
        controller.increase();
        expect(controller.value).toEqual(initValue);
        // set value from parent scope
        parentScope.value = 20;
        parentScope.$digest();
        expect(controller.value).toEqual(20);
    });

    it('setting invalid values', function () {
        // setting value lower than minimum
        parentScope.value = initMin - 1;
        parentScope.$digest();
        expect(controller.value).not.toBeLessThan(initMin);

        // setting value greater than minimum
        parentScope.value = initMax + 1;
        parentScope.$digest();
        expect(controller.value).not.toBeGreaterThan(initMax);

        // setting string value
        parentScope.value = "foo";
        parentScope.$digest();
        expect(typeof controller.value).toEqual('number');
    });

});